@@ -1,7 +1,7 @@
 from advanced_alchemy import base, exceptions, filters, mixins, operations, repository, service, types, utils
 from advanced_alchemy.alembic.commands import AlembicCommands
 from advanced_alchemy.config import AlembicAsyncConfig, AlembicSyncConfig, AsyncSessionConfig, SyncSessionConfig
-from advanced_alchemy.extensions.litestar import providers
+from advanced_alchemy.extensions.litestar import providers, session, store
 from advanced_alchemy.extensions.litestar.cli import get_database_migration_plugin
 from advanced_alchemy.extensions.litestar.dto import SQLAlchemyDTO, SQLAlchemyDTOConfig
 from advanced_alchemy.extensions.litestar.plugins import (
@@ -64,6 +64,8 @@ __all__ = (
     "providers",
     "repository",
     "service",
+    "session",
+    "store",
     "sync_autocommit_before_send_handler",
     "sync_autocommit_handler_maker",
     "sync_default_before_send_handler",
