@@ -505,9 +505,11 @@ def _create_statement_filters(  # noqa: C901
         for field_def in not_in_fields:
             field_def = FieldNameType(name=field_def, type_hint=str) if isinstance(field_def, str) else field_def
 
+            # Capture field_def by value to avoid Python closure late binding gotcha
+            # Without default parameter, all closures would reference the loop variable's final value
             def create_not_in_filter_provider(  # pyright: ignore
-                field_name: FieldNameType,
-            ) -> Callable[..., Optional[NotInCollectionFilter[field_def.type_hint]]]:  # type: ignore
+                field_name: FieldNameType = field_def,  # type: ignore[assignment]
+            ) -> Callable[..., Optional[NotInCollectionFilter[Any]]]:
                 def provide_not_in_filter(  # pyright: ignore
                     values: Optional[list[field_name.type_hint]] = Parameter(  # type: ignore
                         query=camelize(f"{field_name.name}_not_in"), default=None, required=False
@@ -532,9 +534,11 @@ def _create_statement_filters(  # noqa: C901
         for field_def in in_fields:
             field_def = FieldNameType(name=field_def, type_hint=str) if isinstance(field_def, str) else field_def
 
+            # Capture field_def by value to avoid Python closure late binding gotcha
+            # Without default parameter, all closures would reference the loop variable's final value
             def create_in_filter_provider(  # pyright: ignore
-                field_name: FieldNameType,
-            ) -> Callable[..., Optional[CollectionFilter[field_def.type_hint]]]:  # type: ignore # pyright: ignore
+                field_name: FieldNameType = field_def,  # type: ignore[assignment]
+            ) -> Callable[..., Optional[CollectionFilter[Any]]]:
                 def provide_in_filter(  # pyright: ignore
                     values: Optional[list[field_name.type_hint]] = Parameter(  # type: ignore # pyright: ignore
                         query=camelize(f"{field_name.name}_in"), default=None, required=False
