@@ -10,6 +10,7 @@ from functools import cached_property
 from typing import Any, ClassVar, Generic, Optional, Union, cast
 
 from sqlalchemy import Select
+from sqlalchemy import inspect as sa_inspect
 from sqlalchemy.ext.asyncio import AsyncSession
 from sqlalchemy.ext.asyncio.scoping import async_scoped_session
 from sqlalchemy.orm import InstrumentedAttribute
@@ -27,10 +28,12 @@ from advanced_alchemy.repository._util import LoadSpec, model_from_dict
 from advanced_alchemy.repository.typing import MISSING, ModelT, OrderingPair, SQLAlchemyAsyncRepositoryT
 from advanced_alchemy.service._util import ResultConverter
 from advanced_alchemy.service.typing import (
+    UNSET,
     BulkModelDictT,
     ModelDictListT,
     ModelDictT,
     asdict,
+    attrs_nothing,
     is_attrs_instance,
     is_dict,
     is_dto_data,
@@ -448,24 +451,30 @@ class SQLAlchemyAsyncRepositoryReadService(ResultConverter, Generic[ModelT, SQLA
             )
 
         if is_msgspec_struct(data):
-            from msgspec import UNSET
-
             return model_from_dict(
                 model=self.model_type,
-                **{f: val for f in data.__struct_fields__ if (val := getattr(data, f, None)) != UNSET},
+                **{
+                    f: getattr(data, f)
+                    for f in data.__struct_fields__
+                    if hasattr(data, f) and getattr(data, f) is not UNSET
+                },
             )
 
         if is_dto_data(data):
             return cast("ModelT", data.create_instance())
 
         if is_attrs_instance(data):
+            # Filter out attrs.NOTHING values for partial updates
+            def filter_unset(attr: Any, value: Any) -> bool:  # noqa: ARG001
+                return value is not attrs_nothing
+
             return model_from_dict(
                 model=self.model_type,
-                **asdict(data),
+                **asdict(data, filter=filter_unset),
             )
 
         # Fallback for objects with __dict__ (e.g., regular classes)
-        if hasattr(data, "__dict__"):
+        if hasattr(data, "__dict__") and not isinstance(data, self.model_type):
             return model_from_dict(
                 model=self.model_type,
                 **data.__dict__,
@@ -729,33 +738,42 @@ class SQLAlchemyAsyncRepositoryService(
         Returns:
             Updated representation.
         """
-        if is_dict(data) and item_id is not None:
+        # ALWAYS convert data through to_model first to ensure operation hooks are called
+        # This ensures custom to_model() implementations receive the operation="update" parameter
+        # and that to_model_on_update() is properly invoked via the operation_map
+        data = await self.to_model(data, "update")
+
+        if item_id is not None:
+            # When item_id is provided, update existing instance rather than replacing it
+            # This preserves relationships and database-managed fields
             existing_instance = await self.repository.get(
                 item_id, id_attribute=id_attribute, load=load, execution_options=execution_options
             )
-            update_data = await self.to_model_on_update(data)
-            if is_dict(update_data):
-                for key, value in update_data.items():
-                    if getattr(existing_instance, key, MISSING) is not MISSING:
-                        setattr(existing_instance, key, value)
+
+            # Extract attributes from converted model to update existing instance
+            # Only copy attributes that were explicitly set (present in instance state)
+            instance_state = sa_inspect(data)  # pyright: ignore[reportOptionalMemberAccess]
+            for attr in instance_state.mapper.attrs:  # type: ignore[union-attr]  # pyright: ignore[reportOptionalMemberAccess]
+                # Check if attribute was explicitly set in the instance
+                if attr.key in instance_state.dict:  # type: ignore[union-attr]  # pyright: ignore[reportOptionalMemberAccess]
+                    value = getattr(data, attr.key, MISSING)
+                    if value is not MISSING and hasattr(existing_instance, attr.key):
+                        setattr(existing_instance, attr.key, value)
+
             data = existing_instance
-        else:
-            data = await self.to_model(data, "update")
-            if (
-                item_id is None
-                and self.repository.get_id_attribute_value(  # pyright: ignore[reportUnknownMemberType]
-                    item=data,
-                    id_attribute=id_attribute,
-                )
-                is None
-            ):
-                msg = (
-                    "Could not identify ID attribute value.  One of the following is required: "
-                    f"``item_id`` or ``data.{id_attribute or self.repository.id_attribute}``"
-                )
-                raise RepositoryError(msg)
-            if item_id is not None:
-                data = self.repository.set_id_attribute_value(item_id=item_id, item=data, id_attribute=id_attribute)  # pyright: ignore[reportUnknownMemberType]
+        elif (
+            self.repository.get_id_attribute_value(  # pyright: ignore[reportUnknownMemberType]
+                item=data,
+                id_attribute=id_attribute,
+            )
+            is None
+        ):
+            # No item_id provided and no ID on model - error
+            msg = (
+                "Could not identify ID attribute value.  One of the following is required: "
+                f"``item_id`` or ``data.{id_attribute or self.repository.id_attribute}``"
+            )
+            raise RepositoryError(msg)
         return cast(
             "ModelT",
             await self.repository.update(
