@@ -30,29 +30,32 @@ suppress_warnings = [
     "autosectionlabel.*",
     "ref.python",  # TODO: remove when https://github.com/sphinx-doc/sphinx/issues/4961 is fixed
 ]
+
 # -- General configuration ---------------------------------------------------
 extensions = [
-    "sphinx.ext.intersphinx",
+    # Sphinx core extensions
     "sphinx.ext.autodoc",
-    "sphinx.ext.napoleon",
     "sphinx.ext.autosectionlabel",
     "sphinx.ext.githubpages",
+    "sphinx.ext.intersphinx",
+    "sphinx.ext.napoleon",
+    "sphinx.ext.todo",
     "sphinx.ext.viewcode",
+    # Custom extensions
     "tools.sphinx_ext.missing_references",
     "tools.sphinx_ext.changelog",
+    # Third-party extensions
     "sphinx_autodoc_typehints",
     "myst_parser",
     "auto_pytabs.sphinx_ext",
     "sphinx_copybutton",
-    "sphinx.ext.todo",
-    "sphinx.ext.viewcode",
     "sphinx_click",
     "sphinx_toolbox.collapse",
     "sphinx_design",
     "sphinx_togglebutton",
     "sphinx_paramlinks",
 ]
-exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]
+
 intersphinx_mapping = {
     "python": ("https://docs.python.org/3", None),
     "msgspec": ("https://jcristharif.com/msgspec/", None),
@@ -67,14 +70,9 @@ intersphinx_mapping = {
     "sanic": ("https://sanic.readthedocs.io/en/latest/", None),
     "flask": ("https://flask.palletsprojects.com/en/stable/", None),
     "typing_extensions": ("https://typing-extensions.readthedocs.io/en/stable/", None),
+    "attrs": ("https://www.attrs.org/en/stable/", None),
+    "pytest": ("https://docs.pytest.org/en/stable/", None),
 }
-PY_CLASS = "py:class"
-PY_EXC = "py:exc"
-PY_RE = r"py:.*"
-PY_METH = "py:meth"
-PY_ATTR = "py:attr"
-PY_OBJ = "py:obj"
-PY_FUNC = "py:func"
 nitpicky = True
 nitpick_ignore: list[str] = []
 nitpick_ignore_regex: list[str] = []
@@ -94,6 +92,8 @@ autodoc_class_signature = "separated"
 autodoc_default_options = {"special-members": "__init__", "show-inheritance": True, "members": True}
 autodoc_member_order = "bysource"
 autodoc_typehints_format = "short"
+autodoc_typehints = "both"
+autodoc_preserve_defaults = True
 autodoc_type_aliases = {
     "ModelT": "advanced_alchemy.repository.typing.ModelT",
     "FilterTypeT": "advanced_alchemy.filters.FilterTypeT",
@@ -170,7 +170,6 @@ autodoc_mock_imports = [
     "_sa.create_engine._sphinx_paramlinks_creator",
     "sqlalchemy.Dialect",
     "sqlalchemy.orm.MetaData",
-    # Add these new entries:
     "advanced_alchemy.config.engine.EngineConfig",
     "advanced_alchemy.config.asyncio.AsyncSessionConfig",
     "advanced_alchemy.config.sync.SyncSessionConfig",
@@ -196,18 +195,36 @@ copybutton_prompt_text = "$ "
 html_theme = "shibuya"
 html_title = "Advanced Alchemy"
 html_short_title = "AA"
-pygments_style = "dracula"
+
+# Pygments syntax highlighting configuration
+# Using accessible-pygments for WCAG AA/AAA compliant syntax highlighting
+# Light mode: a11y-light provides excellent readability on light backgrounds (WCAG AA)
+# Dark mode: a11y-high-contrast-dark provides maximum readability on dark backgrounds (WCAG AAA)
+pygments_style = "a11y-light"
+pygments_dark_style = "a11y-high-contrast-dark"
+
 todo_include_todos = True
 
 html_static_path = ["_static"]
 html_favicon = "_static/favicon.png"
 templates_path = ["_templates"]
-html_js_files = ["versioning.js"]
-html_css_files = ["custom.css"]
-exclude_patterns = ["_build", "Thumbs.db", ".DS_Store", "PYPI_README.md"]
+html_css_files = ["custom.css", "syntax-highlighting.css"]
+exclude_patterns = ["_build", "Thumbs.db", ".DS_Store", "PYPI_README.md", "guides/**"]
 html_show_sourcelink = True
 html_copy_source = True
 
+# Add SEO-friendly meta tags
+html_meta = {
+    "description": "Advanced Alchemy - A carefully crafted, thoroughly tested, optimized companion library for SQLAlchemy",
+    "keywords": "sqlalchemy, orm, alembic, python, database, litestar, repository-pattern, fastapi, starlette",
+    "author": "Litestar Organization",
+    "og:title": "Advanced Alchemy Documentation",
+    "og:type": "website",
+    "og:description": "Advanced Alchemy - A carefully crafted, thoroughly tested, optimized companion library for SQLAlchemy",
+    "og:site_name": "Advanced Alchemy",
+    "twitter:card": "summary",
+}
+
 html_context = {
     "source_type": "github",
     "source_user": "litestar-org",
