@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 from dataclasses import dataclass
+from functools import cached_property
 from types import ModuleType
 from typing import Annotated, Any, Callable, Optional
 from uuid import UUID
@@ -230,6 +231,124 @@ def test_model_using_func() -> None:
         assert response_callback
 
 
+def test_dto_includes_simple_property() -> None:
+    """Test that @property decorated methods appear in DTO as read-only fields."""
+
+    class ModelWithProperty(Base):
+        __tablename__ = "model_with_property"
+
+        first_name: Mapped[str] = mapped_column()
+        last_name: Mapped[str] = mapped_column()
+
+        @property
+        def full_name(self) -> str:
+            return f"{self.first_name} {self.last_name}"
+
+    config = SQLAlchemyDTOConfig()
+    dto = SQLAlchemyDTO[Annotated[ModelWithProperty, config]]
+    field_defs = list(dto.generate_field_definitions(ModelWithProperty))
+    field_names = {f.name for f in field_defs}
+
+    assert "full_name" in field_names
+
+    full_name_field = next(f for f in field_defs if f.name == "full_name")
+    assert full_name_field.dto_field.mark == Mark.READ_ONLY
+
+
+def test_dto_includes_cached_property() -> None:
+    """Test that @cached_property decorated methods appear in DTO as read-only fields."""
+
+    class ModelWithCachedProperty(Base):
+        __tablename__ = "model_with_cached_property"
+
+        value: Mapped[int] = mapped_column()
+
+        @cached_property
+        def expensive_calculation(self) -> int:
+            return self.value * 2
+
+    config = SQLAlchemyDTOConfig()
+    dto = SQLAlchemyDTO[Annotated[ModelWithCachedProperty, config]]
+    field_defs = list(dto.generate_field_definitions(ModelWithCachedProperty))
+    field_names = {f.name for f in field_defs}
+
+    assert "expensive_calculation" in field_names
+
+    field = next(f for f in field_defs if f.name == "expensive_calculation")
+    assert field.dto_field.mark == Mark.READ_ONLY
+
+
+def test_dto_property_with_setter_is_read_only() -> None:
+    """Test that properties with setters are marked READ_ONLY (setter support not implemented)."""
+
+    class ModelWithPropertySetter(Base):
+        __tablename__ = "model_with_property_setter"
+
+        _internal_value: Mapped[int] = mapped_column(default=0)
+
+        @property
+        def value(self) -> int:
+            return self._internal_value
+
+        @value.setter
+        def value(self, new_value: int) -> None:
+            self._internal_value = new_value
+
+    config = SQLAlchemyDTOConfig()
+    dto = SQLAlchemyDTO[Annotated[ModelWithPropertySetter, config]]
+    field_defs = list(dto.generate_field_definitions(ModelWithPropertySetter))
+    field_names = {f.name for f in field_defs}
+
+    assert "value" in field_names
+
+    field = next(f for f in field_defs if f.name == "value")
+    assert field.dto_field.mark == Mark.READ_ONLY
+
+
+def test_dto_skips_private_properties() -> None:
+    """Test that properties starting with _ are excluded from DTO."""
+
+    class ModelWithPrivateProperty(Base):
+        __tablename__ = "model_with_private_property"
+
+        @property
+        def public_prop(self) -> str:
+            return "public"
+
+        @property
+        def _private_prop(self) -> str:
+            return "private"
+
+    config = SQLAlchemyDTOConfig()
+    dto = SQLAlchemyDTO[Annotated[ModelWithPrivateProperty, config]]
+    field_defs = list(dto.generate_field_definitions(ModelWithPrivateProperty))
+    field_names = {f.name for f in field_defs}
+
+    assert "public_prop" in field_names
+    assert "_private_prop" not in field_names
+
+
+def test_dto_handles_untyped_property() -> None:
+    """Test that properties without type hints are included with Any type."""
+
+    class ModelWithUntypedProperty(Base):
+        __tablename__ = "model_with_untyped_property"
+
+        @property
+        def untyped_prop(self):  # type: ignore[no-untyped-def]
+            return "value"
+
+    config = SQLAlchemyDTOConfig()
+    dto = SQLAlchemyDTO[Annotated[ModelWithUntypedProperty, config]]
+    field_defs = list(dto.generate_field_definitions(ModelWithUntypedProperty))
+    field_names = {f.name for f in field_defs}
+
+    assert "untyped_prop" in field_names
+
+    field = next(f for f in field_defs if f.name == "untyped_prop")
+    assert field is not None
+
+
 def test_dto_with_association_proxy(create_module: Callable[[str], ModuleType]) -> None:
     module = create_module(
         """
