@@ -1,6 +1,10 @@
 """Unit tests for FileObject class."""
 
+import asyncio
+import logging
+import sys
 from pathlib import Path
+from typing import Callable, Optional
 from unittest.mock import AsyncMock, Mock, PropertyMock, patch
 
 import pytest
@@ -8,6 +12,12 @@ import pytest
 from advanced_alchemy.service.typing import PYDANTIC_INSTALLED, BaseModel
 from advanced_alchemy.types.file_object import FileObject
 from advanced_alchemy.types.file_object.base import StorageBackend
+from advanced_alchemy.types.file_object.session_tracker import FileObjectSessionTracker
+
+if sys.version_info >= (3, 11):
+    from builtins import ExceptionGroup
+else:
+    from exceptiongroup import ExceptionGroup  # type: ignore[import-not-found,unused-ignore]
 
 
 def test_file_object_delete_no_backend() -> None:
@@ -622,3 +632,394 @@ def test_pydantic_validation_error_missing_backend() -> None:
     }
     with pytest.raises(ValidationError, match="backend"):  # type: ignore
         FileModel(file=input_dict)  # type: ignore[arg-type]
+
+
+@pytest.mark.asyncio
+async def test_session_tracker_commit_async_reraises_base_exceptions() -> None:
+    """Ensure async commit re-raises non-Exception BaseException instances."""
+    tracker = FileObjectSessionTracker()
+    file_obj = Mock(spec=FileObject)
+    file_obj.path = "tmp"
+    file_obj.save_async = AsyncMock(side_effect=asyncio.CancelledError())
+
+    tracker.add_pending_save(file_obj, b"payload")
+
+    with pytest.raises(asyncio.CancelledError):
+        await tracker.commit_async()
+
+    file_obj.save_async.assert_awaited_once_with(b"payload")
+
+
+def test_session_tracker_commit_ignores_file_not_found_on_delete_sync() -> None:
+    """Sync commit should ignore FileNotFoundError from delete."""
+    tracker = FileObjectSessionTracker()
+    file_obj = Mock(spec=FileObject)
+    file_obj.path = "tmp"
+    file_obj.delete.side_effect = FileNotFoundError()
+
+    tracker.add_pending_delete(file_obj)
+
+    tracker.commit()  # should not raise
+    file_obj.delete.assert_called_once_with()
+
+
+def test_session_tracker_add_pending_delete_ignores_none_path() -> None:
+    """Objects with no path are not added to pending deletes."""
+    tracker = FileObjectSessionTracker()
+    file_obj = Mock(spec=FileObject)
+    file_obj.path = None
+
+    tracker.add_pending_delete(file_obj)
+
+    assert file_obj not in tracker.pending_deletes
+
+
+def test_session_tracker_rollback_ignores_file_not_found_sync() -> None:
+    """Rollback ignores FileNotFoundError from delete in sync path."""
+    tracker = FileObjectSessionTracker()
+    obj = Mock(spec=FileObject)
+    obj.path = "tmp"
+    obj.delete.side_effect = FileNotFoundError()
+
+    tracker._saved_in_transaction.add(obj)
+
+    tracker.rollback()  # should not raise
+    obj.delete.assert_called_once_with()
+
+
+@pytest.mark.asyncio
+async def test_session_tracker_rollback_async_ignores_file_not_found() -> None:
+    """Rollback ignores FileNotFoundError from delete in async path."""
+    tracker = FileObjectSessionTracker()
+    obj = Mock(spec=FileObject)
+    obj.path = "tmp"
+    obj.delete_async = AsyncMock(side_effect=FileNotFoundError())
+
+    tracker._saved_in_transaction.add(obj)
+
+    await tracker.rollback_async()  # should not raise
+    obj.delete_async.assert_awaited_once_with()
+
+
+def test_session_tracker_commit_multiple_saves_then_rollback_deletes_successful_ones() -> None:
+    """Sync commit failure after some saves should allow rollback to delete saved ones."""
+    tracker = FileObjectSessionTracker(raise_on_error=True)
+    obj1 = Mock(spec=FileObject)
+    obj1.path = "p1"
+    obj1.save.return_value = None
+    obj2 = Mock(spec=FileObject)
+    obj2.path = "p2"
+    obj2.save.side_effect = RuntimeError("boom2")
+
+    tracker.add_pending_save(obj1, b"a")
+    tracker.add_pending_save(obj2, b"b")
+
+    with pytest.raises(RuntimeError, match="boom2"):
+        tracker.commit()
+
+    # first save recorded, second failed
+    assert obj1 in tracker._saved_in_transaction
+    assert obj2 not in tracker._saved_in_transaction
+
+    # rollback deletes the saved file
+    tracker.rollback()
+    obj1.delete.assert_called_once_with()
+
+
+@pytest.mark.asyncio
+async def test_session_tracker_commit_async_multiple_saves_raises_first_and_rollback_deletes_success() -> None:
+    """Async commit raises first failure; rollback deletes successful saves."""
+    tracker = FileObjectSessionTracker(raise_on_error=True)
+    ok = Mock(spec=FileObject)
+    ok.path = "ok"
+    ok.save_async = AsyncMock(return_value=None)
+    bad = Mock(spec=FileObject)
+    bad.path = "bad"
+    bad.save_async = AsyncMock(side_effect=RuntimeError("first failure"))
+
+    tracker.add_pending_save(ok, b"ok")
+    tracker.add_pending_save(bad, b"bad")
+
+    with pytest.raises(RuntimeError, match="first failure"):
+        await tracker.commit_async()
+
+    # success recorded despite overall failure
+    assert ok in tracker._saved_in_transaction
+    # rollback deletes the successful one
+    await tracker.rollback_async()
+    ok.delete_async.assert_awaited_once_with()
+
+
+@pytest.mark.asyncio
+async def test_session_tracker_commit_async_raises_first_save_exception_in_order() -> None:
+    """When multiple saves fail, an ExceptionGroup is raised with all failures."""
+    tracker = FileObjectSessionTracker(raise_on_error=True)
+    a = Mock(spec=FileObject)
+    a.path = "a"
+    a.save_async = AsyncMock(side_effect=RuntimeError("first"))
+    b = Mock(spec=FileObject)
+    b.path = "b"
+    b.save_async = AsyncMock(side_effect=RuntimeError("second"))
+
+    tracker.add_pending_save(a, b"x")
+    tracker.add_pending_save(b, b"y")
+
+    # Multiple errors raise ExceptionGroup
+    with pytest.raises(ExceptionGroup, match="multiple FileObject operation failures"):
+        await tracker.commit_async()
+
+
+@pytest.mark.asyncio
+async def test_session_tracker_commit_async_logs_exc_info_on_save_error(caplog: "pytest.LogCaptureFixture") -> None:
+    """Async save errors are logged with exc_info for stack traces."""
+    tracker = FileObjectSessionTracker(raise_on_error=True)
+    obj = Mock(spec=FileObject)
+    obj.path = "tmp"
+    obj.save_async = AsyncMock(side_effect=RuntimeError("fail"))
+
+    tracker.add_pending_save(obj, b"data")
+
+    with caplog.at_level(logging.ERROR):
+        with pytest.raises(RuntimeError):
+            await tracker.commit_async()
+
+    # Find our error record and assert exc_info present
+    err_records = [r for r in caplog.records if "error saving file" in r.message]
+    assert err_records and any(rec.exc_info for rec in err_records)
+
+
+def test_session_tracker_commit_delete_exceptions_sync(
+    caplog: "pytest.LogCaptureFixture",
+) -> None:
+    """Parametrized-like table: sync delete exceptions behavior."""
+    tracker = FileObjectSessionTracker(raise_on_error=True)
+
+    cases = [
+        (FileNotFoundError(), None, False),
+        (RuntimeError("boom"), RuntimeError, True),
+        (asyncio.CancelledError(), asyncio.CancelledError, False),
+    ]
+
+    for exc, expected_exc, expect_log in cases:
+        file_obj = Mock(spec=FileObject)
+        file_obj.path = "tmp"
+        file_obj.delete.side_effect = exc
+        tracker.add_pending_delete(file_obj)
+
+        with caplog.at_level(logging.ERROR):
+            if expected_exc is None:
+                tracker.commit()
+            else:
+                with pytest.raises(expected_exc):
+                    tracker.commit()
+
+        file_obj.delete.assert_called_once_with()
+        if expect_log:
+            assert any("error deleting file" in r.message for r in caplog.records)
+        else:
+            assert not any("error deleting file" in r.message for r in caplog.records)
+
+        # reset tracker for next case
+        tracker.clear()
+        caplog.clear()
+
+
+def test_session_tracker_commit_save_exceptions_sync(caplog: "pytest.LogCaptureFixture") -> None:
+    """Sync save exceptions: RuntimeError logs+raises; BaseException bubbles without log."""
+    tracker = FileObjectSessionTracker(raise_on_error=True)
+
+    cases = [
+        (RuntimeError("sync failure"), RuntimeError, True),
+        (asyncio.CancelledError(), asyncio.CancelledError, False),
+    ]
+
+    for exc, expected_exc, expect_log in cases:
+        file_obj = Mock(spec=FileObject)
+        file_obj.path = "tmp"
+        file_obj.save.side_effect = exc
+        tracker.add_pending_save(file_obj, b"payload")
+
+        with caplog.at_level(logging.ERROR):
+            with pytest.raises(expected_exc):
+                tracker.commit()
+
+        file_obj.save.assert_called_once_with(b"payload")
+        if expect_log:
+            assert any("error saving file" in r.message for r in caplog.records)
+        else:
+            assert not any("error saving file" in r.message for r in caplog.records)
+
+        tracker.clear()
+        caplog.clear()
+
+
+@pytest.mark.asyncio
+@pytest.mark.parametrize("mode", ["sync", "async"])
+async def test_session_tracker_commit_does_not_clear_state_on_error(mode: str) -> None:
+    """Commit keeps pending items when a save fails (sync and async)."""
+    tracker = FileObjectSessionTracker(raise_on_error=True)
+    obj = Mock(spec=FileObject)
+    obj.path = "tmp"
+    if mode == "sync":
+        obj.save.side_effect = RuntimeError("boom")
+        tracker.add_pending_save(obj, b"data")
+        with pytest.raises(RuntimeError):
+            tracker.commit()
+    else:
+        obj.save_async = AsyncMock(side_effect=RuntimeError("boom"))
+        tracker.add_pending_save(obj, b"data")
+        with pytest.raises(RuntimeError):
+            await tracker.commit_async()
+
+    assert obj in tracker.pending_saves
+
+
+@pytest.mark.asyncio
+@pytest.mark.parametrize("mode", ["sync", "async"])
+async def test_session_tracker_commit_clears_state_on_success(mode: str) -> None:
+    """Commit clears state after successful operations (sync and async)."""
+    tracker = FileObjectSessionTracker()
+    save_obj = Mock(spec=FileObject)
+    save_obj.path = "tmp1"
+    del_obj = Mock(spec=FileObject)
+    del_obj.path = "tmp2"
+
+    if mode == "sync":
+        save_obj.save.return_value = None
+        del_obj.delete.return_value = None
+        tracker.add_pending_save(save_obj, b"data")
+        tracker.add_pending_delete(del_obj)
+        tracker.commit()
+    else:
+        save_obj.save_async = AsyncMock(return_value=None)
+        del_obj.delete_async = AsyncMock(return_value=None)
+        tracker.add_pending_save(save_obj, b"data")
+        tracker.add_pending_delete(del_obj)
+        await tracker.commit_async()
+
+    assert not tracker.pending_saves
+    assert not tracker.pending_deletes
+    assert not tracker._saved_in_transaction
+
+
+@pytest.mark.asyncio
+@pytest.mark.parametrize(
+    "mode, expect_delete_called",
+    [("sync", False), ("async", True)],
+)
+async def test_session_tracker_commit_delete_attempt_when_save_fails(mode: str, expect_delete_called: bool) -> None:
+    """Sync commit aborts before deletes; async still attempts deletes on gather."""
+    tracker = FileObjectSessionTracker(raise_on_error=True)
+    save_obj = Mock(spec=FileObject)
+    save_obj.path = "tmp1"
+    del_obj = Mock(spec=FileObject)
+    del_obj.path = "tmp2"
+
+    tracker.add_pending_save(save_obj, b"data")
+    tracker.add_pending_delete(del_obj)
+
+    if mode == "sync":
+        save_obj.save.side_effect = RuntimeError("save failed")
+        with pytest.raises(RuntimeError):
+            tracker.commit()
+        assert del_obj.delete.call_count == (1 if expect_delete_called else 0)
+    else:
+        save_obj.save_async = AsyncMock(side_effect=RuntimeError("save failed"))
+        del_obj.delete_async = AsyncMock(return_value=None)
+        with pytest.raises(RuntimeError):
+            await tracker.commit_async()
+        assert del_obj.delete_async.await_count == (1 if expect_delete_called else 0)
+
+
+@pytest.mark.asyncio
+@pytest.mark.parametrize("mode", ["sync", "async"])
+async def test_session_tracker_rollback_reraises_delete_errors_param(
+    mode: str, caplog: "pytest.LogCaptureFixture"
+) -> None:
+    """Rollback re-raises delete errors in both modes."""
+    tracker = FileObjectSessionTracker()
+    obj = Mock(spec=FileObject)
+    obj.path = "tmp"
+    tracker._saved_in_transaction.add(obj)
+
+    if mode == "sync":
+        obj.delete.side_effect = RuntimeError("rollback delete failure")
+        with caplog.at_level(logging.ERROR):
+            with pytest.raises(RuntimeError, match="rollback delete failure"):
+                tracker.rollback()
+        obj.delete.assert_called_once_with()
+    else:
+        obj.delete_async = AsyncMock(side_effect=RuntimeError("rollback async delete failure"))
+        with caplog.at_level(logging.ERROR):
+            with pytest.raises(RuntimeError, match="rollback async delete failure"):
+                await tracker.rollback_async()
+        obj.delete_async.assert_awaited_once_with()
+
+
+@pytest.mark.parametrize(
+    "ops, expected_in_saves, expected_in_deletes",
+    [
+        (("save", "delete"), False, True),
+        (("delete", "save"), True, False),
+    ],
+)
+def test_session_tracker_override_semantics(
+    ops: "tuple[str, str]",
+    expected_in_saves: bool,
+    expected_in_deletes: bool,
+) -> None:
+    """Parametrized check that save/delete override each other appropriately."""
+    tracker = FileObjectSessionTracker()
+    file_obj = Mock(spec=FileObject)
+    file_obj.path = "tmp"
+
+    first, second = ops
+    if first == "save":
+        tracker.add_pending_save(file_obj, b"data")
+    else:
+        tracker.add_pending_delete(file_obj)
+
+    if second == "save":
+        tracker.add_pending_save(file_obj, b"data")
+    else:
+        tracker.add_pending_delete(file_obj)
+
+    assert (file_obj in tracker.pending_saves) is expected_in_saves
+    assert (file_obj in tracker.pending_deletes) is expected_in_deletes
+
+
+@pytest.mark.asyncio
+@pytest.mark.parametrize(
+    "exc_factory, expected_exception, expect_log",
+    [
+        (lambda: RuntimeError("delete boom"), RuntimeError, True),
+        (lambda: FileNotFoundError(), None, False),
+        (lambda: asyncio.CancelledError(), asyncio.CancelledError, False),
+    ],
+)
+async def test_session_tracker_commit_async_delete_exceptions(
+    caplog: "pytest.LogCaptureFixture",
+    exc_factory: "Callable[[], BaseException]",
+    expected_exception: "Optional[type[BaseException]]",
+    expect_log: bool,
+) -> None:
+    """Parametrized verification of async delete exception handling semantics."""
+    tracker = FileObjectSessionTracker(raise_on_error=True)
+    file_obj = Mock(spec=FileObject)
+    file_obj.path = "tmp"
+    file_obj.delete_async = AsyncMock(side_effect=exc_factory())
+
+    tracker.add_pending_delete(file_obj)
+
+    with caplog.at_level(logging.ERROR):
+        if expected_exception is None:
+            await tracker.commit_async()
+        else:
+            with pytest.raises(expected_exception):
+                await tracker.commit_async()
+
+    file_obj.delete_async.assert_awaited_once_with()
+    if expect_log:
+        assert any("error deleting file" in r.message for r in caplog.records)
+    else:
+        assert not any("error deleting file" in r.message for r in caplog.records)
